% Code for ultrasound tomography database acquisition
% Author: Stefano Franceschini
% date of the current version: 28/04/2022

close all
clear all
clc

%=================================
% Parameters setting
%=================================

numtx=4;       % number of transmitters
Tmax = 0.02;    % time dedicated to each freqency step
fs_dq = 500e3;      %sample frequency  Maximum available DAQ sampling frequency: 1 MSamples/s
f0 = 40e3;          % central frequency of the considered band
c0 = 343;           % speed of sound in air [=] m/s
fmin_stepped = 38e3; % lowest frequency to be transmitted
fmax_stepped = 42e3; % highest frequency to be transmitted
Nfreqs_stepped = 41; % number of frequency steps
num_samples = Tmax*fs_dq;  %number of samples to be acquired by each receiver at each frequency
ff = linspace(fmin_stepped,fmax_stepped,Nfreqs_stepped);  % stepped operative frequencies 
f_ax = linspace(-fs_dq/2,fs_dq/2,num_samples);
ch_list=[0,8,1,9,2,10,4,12,5,21,28,20,27,19,26,18,25,17]; %list of phisical inputs of the ADC
dq = daq('ni');
dq.Rate = fs_dq; 
dt0 = 1/fs_dq; 
f = fgen;
f.Resource = 'USB0::0x0957::0x0407::MY44045212::0'; % physical address of the waveform generator
connect(f)
display(f)
selectChannel(f, '1');
enableOutput(f);
f.Waveform = 'sine';
f.Amplitude = 10;
tx=zeros(length(ff),length(ch_list),num_samples);
rx=zeros(length(ff),length(ch_list),num_samples);
beat=zeros(length(ff),length(ch_list),num_samples);
mean_beat=zeros(length(ch_list),numtx,length(ff));
BPF1 = abs(f_ax)>=fmin_stepped & abs(f_ax)<=fmax_stepped; %digital band pass filter [38,42] kHz

%=================================
% Data acquisition
%=================================

for kk=1:numtx
    for  jj=1:length(ch_list)
        
        ch_tx=addinput(dq, "Dev1", "ai15", "Voltage");
        ch_tx.TerminalConfig = 'SingleEnded';
        ch_rx=addinput(dq, "Dev1", strcat("ai",num2str(ch_list(jj))), "Voltage"); %select which receiver is active
        ch_rx.TerminalConfig = 'SingleEnded';
        
        for ii=1:length(ff)
            f.Frequency = single(ff(ii)); %select the frequency to be transmitted
            f.Offset = 0;
            f.Mode = 'continuous';
            pause(0.2)  %wait the end of the trasient
            [data, timestamps, ~] = read(dq, num_samples, "OutputFormat", "Matrix");
            tx(ii,jj,:)=data(:,1).';
            rx(ii,jj,:)=data(:,2).';

        end
        
    removechannel(dq,2)
    removechannel(dq,1)
    
    end

for  ii=1:length(ff)
    for jj=1:length(ch_list)     %compute mean beat extraction
        
        Tx = squeeze(fftshift(fft(tx(ii,jj,:))));
        Rx = squeeze(fftshift(fft(rx(ii,jj,:))));
        tx_f=real(ifft(ifftshift(BPF1.*Tx.')));
        rx_f=real(ifft(ifftshift(BPF1.*Rx.')));
        tx_h=hilbert(tx_f);
        rx_h=hilbert(rx_f);
        beat(ii,jj,:) =rx_h.*conj(tx_h);
        mean_beat(jj,kk,ii) = mean(beat(ii,jj,:));  %final data (#tx,#rx,#frequencies)
    
    end
end

disp('Change transmitter') %the transmitter is changed via mechanical switch

end
